/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "cmddefine.h"
#include "shellinstance.h"
#include "iosystem.h"

#include "debugnew.h"

namespace shellp
{

CmdDefine::CmdDefine(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pNameArg = new ShellCmdStringArg("name");
	m_pNameArg->setDescription("Name of the preprocessor variable.");
	m_pNameArg->setExpand(false);
	addArgument(m_pNameArg);

	m_pValueArg = new ShellCmdStringArg("value");
	m_pValueArg->setDescription("Content of the preprocessor variable.");
	m_pValueArg->setExpand(false);
	addArgument(m_pValueArg);

	setDescription("Assigns a specific content to a preprocessor variable name. If the name is A, $A will be replaced by the content of A.");
}

CmdDefine::~CmdDefine()
{
	delete m_pNameArg;
	delete m_pValueArg;
}

bool CmdDefine::execute()
{
	ShellInstance *pInst = ShellInstance::getInstance();

	std::string name = m_pNameArg->getValue();
	std::string value = m_pValueArg->getValue();

	if (!pInst->registerPreprocessorVariable(name, value))
	{
		setErrorString("Unable to register this preprocessor variable: " + pInst->getErrorString());
		return false;
	}
	return true;
}

CmdListDefines::CmdListDefines(const std::string &cmdName) : ShellCommand(cmdName)
{
	setDescription("List the names and content of the preprocessor variables currently defined.");
}

CmdListDefines::~CmdListDefines()
{
}

bool CmdListDefines::execute()
{
	ShellInstance *pInst = ShellInstance::getInstance();
	IOSystem *pIOSys = pInst->getIOSystem();

	std::vector<std::string> names;
	pInst->getPreprocessorVariables(names);
	int maxLen = 0;

	for (int i = 0 ; i < names.size() ; i++)
	{
		if (maxLen < names[i].length())
			maxLen = names[i].length();
	}

	for (int i = 0 ; i < names.size() ; i++)
	{
		const std::string &name = names[i];
		int nameLen = name.length();
		int spaceLen = maxLen-nameLen;

		std::string spaces;
		spaces.resize(spaceLen, ' ');

		std::string value;
		pInst->getPreprocessorVariableContent(names[i], value);

		pIOSys->print("  %s%s = %s", name.c_str(), spaces.c_str(), value.c_str());
	}

	return true;
}

} // end namespace
