/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "cmddir.h"
#include "shellinstance.h"
#include "iosystem.h"
#include "dirutil.h"
#include <stdlib.h>

#include "debugnew.h"

namespace shellp
{

CmdChangeDir::CmdChangeDir(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pDirName = new ShellCmdStringArg("directory");
	m_pDirName->setDescription("Directory which should become the current working directory.");
	addArgument(m_pDirName);
	
	setDescription("Change the current working directory.");
}

CmdChangeDir::~CmdChangeDir()
{
	delete m_pDirName;
}

bool CmdChangeDir::execute()
{
	std::string dirName = m_pDirName->getValue();

	if (!changeDirectory(dirName))
	{
		setErrorString("Unable to change to the specified directory");
		return false;
	}

	return true;
}

CmdGetDir::CmdGetDir(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pVarName = new ShellCmdStringArg("varname");
	m_pVarName->setDefault("*");
	m_pVarName->setDescription("If specified, the current directory will be stored in this preprocessor variable.");
	addArgument(m_pVarName);

	setDescription("Display the current working directory, optionally storing it in a preprocessor variable.");
}

CmdGetDir::~CmdGetDir()
{
	delete m_pVarName;
}

bool CmdGetDir::execute()
{
	ShellInstance *pInst = ShellInstance::getInstance();
	IOSystem *m_pIOSys = pInst->getIOSystem();

	std::string dirName = getCurrentDirectory();

	m_pIOSys->writeOutputLine("Current directory is: " + dirName);

	std::string varName = m_pVarName->getValue();
	if (varName != "*") // store it in a variable as well
	{
		if (!pInst->registerPreprocessorVariable(varName, dirName))
		{
			setErrorString("Unable to register this preprocessor variable: " + pInst->getErrorString());
			return false;
		}
	}
	return true;
}

} // end namespace
