/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "shellconfig.h"
#include "ncursesiosystem.h"

#ifdef SHELLP_CONFIG_NCURSES

#include <ncurses.h>
#include <sys/ioctl.h>
#include <string.h>
#include <unistd.h>
#include <iostream>

#include "debugnew.h"

#define KEYCODE_HOME2                  0x01
#define KEYCODE_BREAK                  0x03
#define KEYCODE_END2                   0x05
#define KEYCODE_BACKSPACE              0x08
#define KEYCODE_TAB                    0x09
#define KEYCODE_REDRAW                 0x0C
#define KEYCODE_ENTER                  0x0D
#define KEYCODE_ESCAPE                 0x1B
#define KEYCODE_UP                     0x101
#define KEYCODE_DOWN                   0x102
#define KEYCODE_LEFT                   0x103
#define KEYCODE_RIGHT                  0x104
#define KEYCODE_CENTER                 0x105
#define KEYCODE_INSERT                 0x106
#define KEYCODE_DELETE                 0x107
#define KEYCODE_HOME                   0x108
#define KEYCODE_END                    0x109
#define KEYCODE_PGUP                   0x10A
#define KEYCODE_PGDN                   0x10B
#define KEYCODE_F1                     0x10C
#define KEYCODE_F2                     0x10D
#define KEYCODE_F3                     0x10E
#define KEYCODE_F4                     0x10F
#define KEYCODE_F5                     0x110
#define KEYCODE_F6                     0x111
#define KEYCODE_F7                     0x112
#define KEYCODE_F8                     0x113
#define KEYCODE_F9                     0x114
#define KEYCODE_F10                    0x115
#define KEYCODE_F11                    0x116
#define KEYCODE_F12                    0x117

namespace shellp
{

NCursesIOSystem::NCursesIOSystem()
{
	setEcho(false);
	stdscr = 0;
}

NCursesIOSystem::~NCursesIOSystem()
{
	if (stdscr != 0)
		endwin();
}

bool NCursesIOSystem::init(bool useCBreakInsteadOfRaw)
{
	stdscr = initscr();
	if (stdscr == 0)
	{
		setErrorString("Can't initialize ncurses");
		return false;
	}
			
	if (useCBreakInsteadOfRaw)
		cbreak();
	else
		raw();
	scrollok((WINDOW *)stdscr,true);
	nonl();

	return true;
}

#define MAXCMDLEN 16384

void NCursesIOSystem::writeInputPrefix(const std::string &prefix)
{
	msg = prefix;
	msglen = strlen(msg.c_str());

	waddnstr((WINDOW *)stdscr, msg.c_str(), -1);
	wrefresh((WINDOW *)stdscr);
}

bool NCursesIOSystem::readInputLineInternal(std::string &line)
{
	uint32_t key;
	std::list<std::string>::const_iterator it = cmdlinehistory.end();
	std::string cmdbackup;
	int cols = COLS;
	char str[MAXCMDLEN];

	int starty,startx;
	int i;

	getyx((WINDOW *)stdscr,starty,startx);
		
	str[0] = 0;
	int len = 0;
	int pos = 0;
	int offset = 0;
		
	while ((key = GetKey()) != KEYCODE_ENTER)
	{
		if (key < 127 && key > 31) // normal character
		{
			if (pos < MAXCMDLEN-2)
			{
				// insert character
				int i;
				for (i = len-1 ; i >= pos ; i--)
				{
					str[i+1] = str[i];
				}
				len++;
				str[len] = 0;
				str[pos] = key;
				pos++;

				if ((pos-offset) == cols-1-msglen)
					offset += (cols-1-msglen)/2;
			}
		}
		else // special keys
		{
			if (key == KEYCODE_LEFT)
			{
				if (pos > 0)
				{
					pos--;
					if (offset > 0 && (pos-offset) == 0)
						offset -= (cols-1-msglen)/2;
				}
			}
			else if (key == KEYCODE_RIGHT)
			{
				if (pos < len)
				{
					pos++;
					if ((pos-offset) == cols-1-msglen)
						offset += (cols-1-msglen)/2;
				}
			}
			else if (key == KEYCODE_BACKSPACE)
			{
				if (pos > 0)
				{
					for (i = pos-1 ; i < len-1 ; i++)
						str[i] = str[i+1];
					len--;
					str[len] = 0;
					pos--;
					if (offset > 0 && (pos-offset) == 0)
						offset -= (cols-1-msglen)/2;
				}
			}
			else if (key == KEYCODE_DELETE)
			{
				if (pos < len)
				{
					for (i = pos ; i < len-1 ; i++)
						str[i] = str[i+1];
					len--;
					str[len] = 0;
				}
			}
			else if (key == KEYCODE_REDRAW)
			{
				int oldpos = pos;
				
				clear();
				pos = 0;
				offset = 0;
				cols = COLS;
				waddnstr((WINDOW *)stdscr, msg.c_str(), -1);
				refresh();
				getyx((WINDOW *)stdscr,starty,startx);
				while (pos != oldpos)
				{
					pos++;
					if ((pos-offset) == cols-1-msglen)
						offset += (cols-1-msglen)/2;
				}
			}
			else if (key == KEYCODE_HOME || key == KEYCODE_HOME2)
			{
				pos = 0;
				offset = 0;
			}
			else if (key == KEYCODE_END || key == KEYCODE_END2)
			{
				offset = 0;
				pos = 0;
				while (pos != len)
				{
					pos++;
					if ((pos-offset) == cols-1-msglen)
						offset += (cols-1-msglen)/2;
				}
			}
			else if (key == KEYCODE_UP)
			{
				if (it != cmdlinehistory.begin())
				{
					if (it == cmdlinehistory.end())
						cmdbackup = std::string(str);
					it--;
					
					strcpy(str,(*it).c_str());
					len = strlen(str);
					pos = 0;
					offset = 0;
					while (pos != len)
					{
						pos++;
						if ((pos-offset) == cols-1-msglen)
							offset += (cols-1-msglen)/2;
					}
				}
			}
			else if (key == KEYCODE_DOWN)
			{
				if (it != cmdlinehistory.end())
				{
					it++;
					if (it == cmdlinehistory.end())
						strcpy(str,cmdbackup.c_str());
					else
						strcpy(str,(*it).c_str());

					len = strlen(str);
					pos = 0;
					offset = 0;
					while (pos != len)
					{
						pos++;
						if ((pos-offset) == cols-1-msglen)
							offset += (cols-1-msglen)/2;
					}
				}
			}
		}
		wmove((WINDOW *)stdscr,starty,startx);
		waddnstr((WINDOW *)stdscr,str+offset,cols-1-msglen);
		for (i = len-offset+msglen ; i < cols ; i++)
			waddch((WINDOW*)stdscr,' ');
		wmove((WINDOW *)stdscr,starty,startx+pos-offset);
		wrefresh((WINDOW *)stdscr);
	}
	if (starty == LINES-2)
	{
		scroll((WINDOW *)stdscr);
		wmove((WINDOW *)stdscr,starty,0);
	}
	else
		wmove((WINDOW *)stdscr,starty+1,0);

	bool found = false;
	for (i = 0 ; !found && i < len ; i++)
	{
		if (str[i] != ' ')
			found = true;
	}
	if (found)
		cmdlinehistory.push_back(std::string(str));
	
	line = std::string(str);
	return true;
}

void NCursesIOSystem::writeOutputLine(const std::string &line, bool newline)
{
	std::string::size_type startPos = 0;
	bool done = false;

	do
	{
		std::string::size_type findPos = line.find("\n", startPos);
		std::string::size_type length = line.length() - startPos;

		if (findPos != std::string::npos)
			length = findPos-startPos;

		std::string subStr = line.substr(startPos, length);

		if (findPos == std::string::npos)
		{
			done = true;
			writeSingleOutputLine(subStr, newline);
		}
		else
		{
			writeSingleOutputLine(subStr, true);
			startPos = findPos+1;
		}

	} while (!done);
}

void NCursesIOSystem::writeSingleOutputLine(const std::string &line, bool newline)
{
	if (stdscr != 0)
	{
		if (!newline)
		{	
			int x,y;
			getyx((WINDOW*)stdscr,y,x);
		
			waddnstr((WINDOW*)stdscr,line.c_str(),COLS-x);
			wmove((WINDOW*)stdscr,y,x);
		}
		else
		{
			int x,y;
			const char *str = line.c_str();
			int len = line.size();
			
			while (len >= 0)
			{
				getyx((WINDOW*)stdscr,y,x);
		
				int num = COLS-x;
				int newx = x;

				if (num >= len)
				{
					num = len;
					newx = 0;
				}
				waddnstr((WINDOW*)stdscr,str,num);
				str += num;
				len -= num;
				
				if (y == LINES-2)
				{
					scroll((WINDOW*)stdscr);
					wmove((WINDOW*)stdscr,y,newx);
				}
				else
					wmove((WINDOW*)stdscr,y+1,newx);
			
				if (len == 0)
					len = -1;
			}
		}
		wrefresh((WINDOW*)stdscr);
	}
}

uint32_t NCursesIOSystem::valuetable[NCURSESIOSYSTEM_NUM_KEYVALUES][3]={
                                         {0x41,0,KEYCODE_UP},
                                         {0x42,0,KEYCODE_DOWN},
                                         {0x44,0,KEYCODE_LEFT},
                                         {0x43,0,KEYCODE_RIGHT},
                                         {0x47,0,KEYCODE_CENTER},
					 {0x48,0,KEYCODE_HOME},
					 {0x46,0,KEYCODE_END},
                                         {0x7E32,0,KEYCODE_INSERT},
                                         {0x7E33,0,KEYCODE_DELETE},
                                         {0x7E31,0,KEYCODE_HOME},
                                         {0x7E34,0,KEYCODE_END},
                                         {0x7E35,0,KEYCODE_PGUP},
                                         {0x7E36,0,KEYCODE_PGDN},
                                         {0x415B,0x7E3832,KEYCODE_F1},
                                         {0x425B,0x7E3333,KEYCODE_F2},
                                         {0x435B,0x7E3433,KEYCODE_F3},
                                         {0x445B,0x7E3932,KEYCODE_F4},
                                         {0x455B,0x7E3233,KEYCODE_F5},
                                         {0x7E3731,0,KEYCODE_F6},
                                         {0x7E3831,0,KEYCODE_F7},
                                         {0x7E3931,0,KEYCODE_F8},
                                         {0x7E3032,0,KEYCODE_F9},
                                         {0x7E3132,0,KEYCODE_F10},
                                         {0x7E3332,0,KEYCODE_F11},
                                         {0x7E3432,0,KEYCODE_F12}};

uint32_t NCursesIOSystem::GetKey()
{
	int number,i;
	bool found;
	uint8_t keyarray[4];
	uint8_t inputarray[10];
	uint32_t value;
	uint32_t key,key2;

	key = 0;
	read(0,(char*)&key,1); /* Get the fist key */
	if (key == 0x1B) /* Possibly an escape sequence */
	{
		ioctl(0,FIONREAD,&number); 
		if (number) 
		{
			number = read(0,inputarray,10);
			if (number >= 2)  /* There was data following the ESC-char */
			{
				if (inputarray[0] == 0x5B && number <= 5) /* It's an ansi code */
				{
					for (i = 0 ; i < number-1 ; i++)
						keyarray[i] = inputarray[i+1];
					
					for (i = number-1 ; i < 4 ; i++)
						keyarray[i] = 0;

					value = (uint32_t)keyarray[0];
					value += (uint32_t)keyarray[1] << 8;
					value += (uint32_t)keyarray[2] << 16;
					value += (uint32_t)keyarray[3] << 24;
					
					found = false;
					i = 0;

					// Search for the value in the valuetables

					while (i < NCURSESIOSYSTEM_NUM_KEYVALUES && !found)
					{
						if (value == valuetable[i][0]) 
							found = true;
						else if (value == valuetable[i][1]) 
							found = true;
						else 
							i++;
					}
					
					if (found)
						key = (uint32_t)valuetable[i][2];
					else
						key = 0; 
				}
				else
					key = 0;
			}
		}
	}
	else // It was not ESC that was pressed, and not an escape sequence
	{
		key2 = key;
		switch (key2)
		{
		case 1:   
			key = KEYCODE_HOME;
			break;
		case 4:   
			key = KEYCODE_DELETE;
			break;
		case 5:   
			key = KEYCODE_END;
			break;
		case 22:  
			key = KEYCODE_PGDN;
			break;
		case 21:  
			key = KEYCODE_PGUP;
			break;
		case 26:  
			key = KEYCODE_BREAK;
			break;
		case 127: 
			key = KEYCODE_BACKSPACE;
			break;
		}
	}
	return key;
}

} // end namespace

#else

namespace shellp
{

NCursesIOSystem::NCursesIOSystem()
{
}

NCursesIOSystem::~NCursesIOSystem()
{
}

bool NCursesIOSystem::init(bool dummy)
{
	setErrorString("No ncurses implementation available on this platform");
	return false;

}

bool NCursesIOSystem::readInputLineInternal(std::string &line)
{
	setErrorString("No ncurses implementation available on this platform");
	return false;
}

void NCursesIOSystem::writeOutputLine(const std::string &line, bool newline)
{
}

void NCursesIOSystem::writeInputPrefix(const std::string &prefix)
{
}

} // end namespace

#endif // SHELLP_CONFIG_NCURSES

