/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "shell2dplotwindow.h"
#include "qwt/qwt_plot.h"
#include "qwt/qwt_plot_panner.h"
#include "qwt/qwt_plot_magnifier.h"
#include "qwt/qwt_plot_curve.h"
#include "qwt/qwt_plot_canvas.h"
#include "qwt/qwt_scale_engine.h"
#include <QVBoxLayout>
#include <QToolBar>
#include <QAction>

namespace shellp
{

Shell2DPlotWindow::Shell2DPlotWindow(QWidget *pParent)
{
	m_pRequest = 0;
	m_plotPixels = false;
	m_log = false;

	QVBoxLayout *pLayout = new QVBoxLayout(this);

	QToolBar *pBar = new QToolBar(this);
	pBar->show();

	QAction *pAction = pBar->addAction("Toggle linear/log scale");
	QObject::connect(pAction, SIGNAL(triggered()), this, SLOT(toggleLogLinear()));

	pAction = pBar->addAction("Toggle physical/pixels");
	QObject::connect(pAction, SIGNAL(triggered()), this, SLOT(togglePixelsOrPhysical()));

	m_pPlot = new QwtPlot(this);
	pLayout->addWidget(pBar);
	pLayout->addWidget(m_pPlot);
	//pLayout->setContentsMargins(0, 0, 0, 0);

//	(void) new QwtPlotPanner(m_pPlot->canvas());
//	(void) new QwtPlotMagnifier(m_pPlot->canvas());

	m_pPlot->setAutoFillBackground(true);
	m_pPlot->canvas()->setLineWidth( 1 );
	m_pPlot->canvas()->setFrameStyle( QFrame::Box | QFrame::Plain );
	m_pPlot->canvas()->setBorderRadius( 15 );

	QPalette canvasPalette( Qt::white );
	canvasPalette.setColor( QPalette::Foreground, QColor( 133, 190, 232 ) );

	m_pPlot->canvas()->setPalette( canvasPalette );

	m_pPlotCurve = new QwtPlotCurve();
	m_pPlotCurve->attach(m_pPlot);

	resize(500, 500);

	hide();
}

Shell2DPlotWindow::~Shell2DPlotWindow()
{
	if (m_pRequest)
		delete m_pRequest;
}

void Shell2DPlotWindow::processRequest(Shell2DPlotRequest *pRequest)
{
	if (m_pRequest)
		delete m_pRequest;

	m_pRequest = pRequest;

	plotCurrentRequest();

	if (m_pRequest->getForceShow())
		show();
}

void Shell2DPlotWindow::plotCurrentRequest()
{
	std::vector<double> pixelPoints;
	const double *pXPoints = &(m_pRequest->getXPoints()[0]);
	const double *pYPoints = &(m_pRequest->getYPoints()[0]);
	int numPoints = m_pRequest->getXPoints().size();

	if (m_plotPixels)
	{
		pixelPoints.resize(numPoints);

		for (int i = 0 ; i < numPoints ; i++)
			pixelPoints[i] = i;

		pXPoints = &(pixelPoints[0]);
	}

	m_pPlotCurve->setSamples(pXPoints, pYPoints, numPoints);

	m_pPlot->setAxisTitle(QwtPlot::xBottom, m_pRequest->getXLabel().c_str());
	m_pPlot->setAxisTitle(QwtPlot::yLeft, m_pRequest->getYLabel().c_str());
	m_pPlot->setTitle(m_pRequest->getTitle().c_str());
	setWindowTitle(QString("2D Plot: ") + QString(m_pRequest->getTitle().c_str()));

	if (m_log)
		m_pPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
	else
		m_pPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLinearScaleEngine);

	m_pPlot->replot();
}

void Shell2DPlotWindow::toggleLogLinear()
{
	m_log = !m_log;

	if (m_log)
		m_pPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
	else
		m_pPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLinearScaleEngine);

	m_pPlot->replot();
}

void Shell2DPlotWindow::togglePixelsOrPhysical()
{
	m_plotPixels = !m_plotPixels;

	plotCurrentRequest();
}

} // end namespace
