/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "shellcmdarg.h"
#include "shellinstance.h"
#include <stdio.h>
#include <string.h>

// TODO
#include <iostream>


#include "debugnew.h"

namespace shellp
{

bool ShellCmdArg::parseArgumentList(std::vector<ShellCmdArg *> &args, 
		                         const std::vector<std::string> &argStrings,
				         std::string &errStr)
{
	if (argStrings.size() > args.size())
	{
		errStr = std::string("Too many arguments");
		return false;
	}

	std::vector<std::string> newArgStrings;
	
	// check if necessary default values are present
	
	int argNr = 1;
	int i = 0;
	
	for (i = 0 ; i < argStrings.size() ; i++)
	{
		if (argStrings[i] == std::string("*")) // means: use default argument
		{
			if (args[i]->hasDefault())
			{
				newArgStrings.push_back(args[i]->getDefault());
			}
			else
			{
				char str[256];

				// snprintf(str, 256, "%d", argNr);
				sprintf(str, "%d", argNr);
				errStr = std::string("Argument ") + std::string(str) + std::string(" (") 
					+ args[i]->getName() + std::string(") does not have a default value");
				return false;
			}
		}
		else
			newArgStrings.push_back(argStrings[i]);

		argNr++;
	}

	// check if remaining unspecified parameters all have defaults
	
	for ( /* */ ; i < args.size() ; i++)
	{
		if (args[i]->hasDefault())
		{
			newArgStrings.push_back(args[i]->getDefault());
		}
		else
		{
			char str[256];

			//snprintf(str, 256, "%d", argNr);
			sprintf(str, "%d", argNr);
			errStr = std::string("Argument ") + std::string(str) + std::string(" (") 
				+ args[i]->getName() + std::string(") does not have a default value");
			return false;
		}
		argNr++;
	}

	// ok, now the newArgStrings contains all the strings which should be interpreted
	
	argNr = 1;
	
	for (i = 0 ; i < newArgStrings.size() ; i++)
	{
		std::string expandedArg;

		if (args[i]->shouldExpand())
		{
			if (!expandArgument(newArgStrings[i], expandedArg, errStr, 0))
			{
				char str[256];

				sprintf(str, "%d", argNr);
				errStr = std::string("Error parsing argument ") + std::string(str) + std::string(" (") 
					+ args[i]->getName() + std::string("): ") + errStr;
				return false;
			}
		}
		else
			expandedArg = newArgStrings[i];

		if (!args[i]->parse(expandedArg))
		{
			char str[256];

			//snprintf(str, 256, "%d", argNr);
			sprintf(str, "%d", argNr);
			errStr = std::string("Error parsing argument ") + std::string(str) + std::string(" (") 
				+ args[i]->getName() + std::string("): ") + args[i]->getErrorString();
			return false;
		}
		argNr++;
	}
	
	return true;
}

std::string ShellCmdArg::getArgumentNames(const std::vector<ShellCmdArg *> &args)
{
	std::string str;
	
	if (args.size() > 0)
	{
		str += args[0]->getName();
		
		for (int i = 1 ; i < args.size() ; i++)
			str += std::string(" ") + args[i]->getName();
	}
	return str;
}

const char ShellCmdArg::m_pValidChars[]="_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

bool ShellCmdArg::expandArgument(const std::string &arg, std::string &expandedArg, std::string &errorString, int iteration)
{
	if (iteration > 64)
	{
		errorString = std::string("Maximum recursion level reached during preprocessor variable expansion");
		return false;
	}

	std::string result;
	std::string::size_type prevPos = 0;
	std::string::size_type newPos = 0;

	newPos = arg.find("$", prevPos);
	while (newPos != std::string::npos)
	{
		if (newPos >= arg.size()-1)
		{
			errorString = std::string("Found a '$' sign indicating the start of a preprocessor variable at the end of an argument");
			return false;
		}

		std::string::size_type endVarPos;

		endVarPos = arg.find_first_not_of(m_pValidChars, newPos+1);
		if (endVarPos == newPos+1)
		{
			if (arg[endVarPos] == '$') // $$ is used as $
			{
				result += arg.substr(prevPos, newPos+1-prevPos);
				prevPos = newPos+2;
			}
			else
			{
				errorString = std::string("Found a '$' during expansion which is not followed by a characted valid for a preprocessor variable name");
				return false;
			}
		}
		else
		{
			result += arg.substr(prevPos, newPos-prevPos);

			std::string varName;

			if (endVarPos != std::string::npos)
				varName = arg.substr(newPos+1, endVarPos-newPos-1);
			else
				varName = arg.substr(newPos+1);


			ShellInstance *pInstance = ShellInstance::getInstance();
			std::string replaceString;

			if (pInstance == 0)
			{
				errorString = std::string("Encountered a preprocessor variable name, but no shell instance has been created yet to look up its meaning");
				return false;
			}
			if (!pInstance->getPreprocessorVariableContent(varName, replaceString))
			{
				errorString = std::string("Unable to expand preprocessor variable '") + varName + std::string("': ") + pInstance->getErrorString();
				return false;
			}

			std::string expandedReplaceString;

			if (!expandArgument(replaceString, expandedReplaceString, errorString, iteration+1))
				return false;

			result += expandedReplaceString;
			
			prevPos = endVarPos;
		}

		newPos = arg.find("$", prevPos);
	}

	if (prevPos != std::string::npos)
		result += arg.substr(prevPos);

	expandedArg = result;
	return true;
}

} // end namespace
