/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#ifndef SHELLP_SHELLMAINWIN_H

#define SHELLP_SHELLMAINWIN_H

#include "shellconfig.h"
#include <errut/errorbase.h>
#include <jthread/jmutex.h>
#include <QMainWindow>
#include <QThread>
#include <QPalette>
#include <map>

class QTextEdit;
class QLineEdit;
class QProgressBar;

namespace shellp
{

class Shell2DPlotWindow;
class Shell2DPlotRequest;
class Shell3DPlotWindow;
class Shell3DPlotRequest;

class ShellMainWinThread : public QThread
{
	Q_OBJECT
public:
	ShellMainWinThread(const std::string &shellPrefix, QObject *pParent = 0);
	~ShellMainWinThread();

	void run();
private:
	std::string m_shellPrefix;
};

class ShellMainWin : public QMainWindow, public errut::ErrorBase
{
	Q_OBJECT
public:
	ShellMainWin(const std::string &title, const std::string &shellPrefix);
	~ShellMainWin();

	void startThread();
	void requestWriteOutputLine(const std::string &line, bool newline);
	void requestNewPercentage(double pct);
	void waitingForInput();

	bool requestPlot2D(int id, const std::string &title,
			    const std::string &xLabel, const std::string &yLabel,
			    const std::vector<double> &xPoints, const std::vector<double> &yPoints,
			    bool forceShow);
	bool requestPlot3D(int id, const std::string &title, 
			  const std::string &xLabel, const std::string &yLabel,
			  const std::string &zLabel,
			  double xOff, double yOff, double xDist, double yDist,
			  double xMultiplier, double yMultiplier, double zMultiplier,
			  const std::vector<double> &data, int numX, int numY, bool forceShow);
	void requestStatusLine(const std::string &line);
signals:
	void signalWriteOutputLine(const QString &line, bool newline);
	void signalWaitingForInput();
	void signalProgressBarValue(int val);
	void signalNew2DPlotRequest();
	void signalNew3DPlotRequest();
	void signalStatusLine(const QString &line);
	void signalCloseAll();
public slots:
	void lineEntered();
	void writeOutputLine(const QString &line, bool newline);
	void slotWaitingForInput();
	void slotForceQuit();
	void slotNew2DPlotRequest();
	void slotNew3DPlotRequest();
	void slotInterrupt();
	void slotStatusLine(const QString &line);
	void slotThreadFinished();
	void slotSelectFile();
	void slotSelectDir();
private:
	void closeEvent(QCloseEvent *pEvent);
	void insertIntoLineEdit(const QString &s);

	QTextEdit *m_pTextEdit;
	QLineEdit *m_pLineEdit;
	QProgressBar *m_pPctBar;
	ShellMainWinThread *m_pThread;

	jthread::JMutex m_plotRequestMutex;
	std::list<Shell2DPlotRequest *> m_plot2DRequestQueue;
	std::list<Shell3DPlotRequest *> m_plot3DRequestQueue;

	std::map<int, Shell2DPlotWindow *> m_2DPlots;
	std::map<int, Shell3DPlotWindow *> m_3DPlots;
};

} // end namespace

#endif // SHELLP_SHELLMAINWIN_H
